<?php
/**
 * @package         FirePlugins Framework
 * @version         1.0.20
 * 
 * @author          FirePlugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2021 FirePlugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework\Base;

if (!defined('ABSPATH'))
{
	exit; // Exit if accessed directly.
}

abstract class Block
{
	/**
	 * Block namespace.
	 * 
	 * @var  string
	 */
	protected $namespace;
	
	/**
	 * Block identifier.
	 * 
	 * @var  string
	 */
	protected $name;
	
	/**
	 * Front-end style.
	 * 
	 * @var  string
	 */
	protected $style;
	
	/**
	 * Front-end script.
	 * 
	 * @var  string
	 */
	protected $script;
	
	/**
	 * Editor style.
	 * 
	 * @var  string
	 */
	protected $editor_style;
	
	/**
	 * Editor script.
	 * 
	 * @var  string
	 */
	protected $editor_script;
	
	/**
	 * Block attributes.
	 * 
	 * @var  array
	 */
	protected $attributes;
	
	/**
	 * Callback method name that will be used to render the block.
	 * 
	 * @var  string
	 */
	protected $render_callback;
	
	/**
	 * Category.
	 * 
	 * @var  string
	 */
	protected $category;
	
	/**
	 * Keywords that will be used in search results.
	 * 
	 * @var  array
	 */
	protected $keywords;
	
	public function __construct()
	{
		add_action('enqueue_block_editor_assets', [$this, 'assets'], 10);
	}
	
	/**
	 * Register the block.
	 * 
	 * @return  void
	 */
	public function register()
	{
		register_block_type(
			$this->getNamespace() . '/' . $this->getName(),
			$this->getPayload()
		);
	}

	/**
	 * Returns the block payload.
	 * 
	 * @return  array
	 */
	private function getPayload()
	{
		$payload = [];

		if ($this->style)
		{
			$payload['style'] = $this->style;
		}

		if ($this->script)
		{
			$payload['script'] = $this->script;
		}

		if ($this->editor_style)
		{
			$payload['editor_style'] = $this->editor_style;
		}

		if ($this->editor_script)
		{
			$payload['editor_script'] = $this->editor_script;
		}

		if ($this->attributes)
		{
			$payload['attributes'] = $this->attributes;
		}

		if ($this->category)
		{
			$payload['category'] = $this->category;
		}

		if ($this->keywords)
		{
			$payload['keywords'] = $this->keywords;
		}

		if ($this->render_callback && method_exists($this, $this->render_callback))
		{
			$payload['render_callback'] = [$this, $this->render_callback];
		}
		
		return $payload;
	}

	/**
	 * Returns the block namespace.
	 * 
	 * @return  string
	 */
	protected function getNamespace()
	{
		return $this->namespace;
	}

	/**
	 * Returns the block name.
	 * 
	 * @return  string
	 */
	protected function getName()
	{
		return $this->name;
	}

	/**
	 * Overriden by child classes to register their assets.
	 * 
	 * @return  mixed
	 */
	public function assets()
	{
		return true;
	}
}