<?php
/**
 * @package         FirePlugins Framework
 * @version         1.0.20
 * 
 * @author          FirePlugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2021 FirePlugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework\Base\Widgets;

if (!defined('ABSPATH'))
{
	exit; // Exit if accessed directly.
}

/**
 * Countdown
 */
class Countdown extends Widget
{
	/**
	 * Widget default options
	 *
	 * @var array
	 */
	protected $widget_options = [
		/**
		 * The Countdown type:
		 * 
		 * - static: Counts down to a specific date and time. Universal deadline for all visitors.
		 * - dynamic: Set-and-forget solution. The countdown starts when your visitor sees the offer.
		 */
		'countdown_type' => 'static',

		// The Static Countdown Date
		'value' => '',

		/**
		 * The timezone that will be used.
		 * 
		 * - server - Use server's timezone
		 * - client - Use client's timezone
		 */
		'timezone' => 'server',

		// Dynamic Days
		'dynamic_days' => 0,

		// Dynamic Hours
		'dynamic_hours' => 0,

		// Dynamic Minutes
		'dynamic_minutes' => 0,

		// Dynamic Seconds
		'dynamic_seconds' => 0,
		
		/**
		 * The countdown format.
		 * 
		 * Available tags:
		 * {years}
		 * {months}
		 * {days}
		 * {hours}
		 * {minutes}
		 * {seconds}
		 */
		'format' => '{days} days, {hours} hours, {minutes} minutes and {seconds} seconds',

		/**
		 * The countdown theme.
		 * 
		 * Available themes:
		 * theme1
		 * theme2
		 * theme3
		 */
		'theme' => 'theme1',

		/**
		 * Set the action once countdown finishes.
		 * 
		 * Available values:
		 * keep 	- Keep the countdown visible
		 * hide 	- Hide the countdown
		 * message	- Show a message
		 * redirect	- Redirect to a URL
		 */
		'countdown_action' => 'keep',

		/**
		 * The message appearing after the countdown has finished.
		 * 
		 * Requires `countdown_action` to be set to `message`
		 * 
		 * Example: Countdown finished.
		 */
		'finish_text' => '',

		/**
		 * The redirect URL once the countdown expires.
		 * 
		 * Requires `countdown_action` to be set to `redirect`
		 */
		'redirect_url' => '',

		// Whether to display Days
		'days' => true,

		// Days Label
		'days_label' => 'Days',
		
		// Whether to display Hours
		'hours' => true,

		// Hours Label
		'hours_label' => 'Hours',
		
		// Whether to display Minutes
		'minutes' => true,

		// Minutes Label
		'minutes_label' => 'Minutes',
		
		// Whether to display Seconds
		'seconds' => true,

		// Seconds Label
		'seconds_label' => 'Seconds'
	];

	/**
	 * Class constructor
	 *
	 * @param array $options
	 */
	public function __construct($options = [])
	{
		parent::__construct($options);

		$this->prepare();
	}

	/**
	 * Prepares the countdown.
	 * 
	 * @return  void
	 */
	private function prepare()
	{
		$this->options['css_class'] .= ' ' . $this->options['theme'];

		if (!empty($this->options['value']) && $this->options['value'] !== '0000-00-00 00:00:00')
		{
			if ($this->options['countdown_type'] === 'static' && $this->options['timezone'] === 'server')
			{
				// Convert given date time to UTC
				$this->options['value'] = get_gmt_from_date($this->options['value'], 'c');
				
				// get timezone
				$tz = new \DateTimeZone(wp_timezone()->getName());

				// Apply server timezone
				$this->options['value'] = (new \DateTime($this->options['value']))->setTimezone($tz)->format('c');
			}
		}

		// Set countdown payload
		$payload = [
			'data-countdown-type="' . esc_attr($this->options['countdown_type']) . '"',
			'data-value="' . esc_attr($this->options['value']) . '"',
			'data-timezone="' . esc_attr($this->options['timezone']) . '"',
			'data-dynamic-days="' . esc_attr($this->options['dynamic_days']) . '"',
			'data-dynamic-hours="' . esc_attr($this->options['dynamic_hours']) . '"',
			'data-dynamic-minutes="' . esc_attr($this->options['dynamic_minutes']) . '"',
			'data-dynamic-seconds="' . esc_attr($this->options['dynamic_seconds']) . '"',
			'data-finish-text="' . esc_attr(htmlspecialchars($this->options['finish_text'])) . '"',
			'data-redirect-url="' . esc_attr($this->options['redirect_url']) . '"',
			'data-theme="' . esc_attr($this->options['theme']) . '"',
			'data-format="' . esc_attr(htmlspecialchars($this->options['format'])) . '"',
			'data-countdown-action="' . esc_attr($this->options['countdown_action']) . '"',
			'data-days="' . esc_attr(var_export($this->options['days'], true)) . '"',
			'data-days-label="' . esc_attr($this->options['days_label']) . '"',
			'data-hours="' . esc_attr(var_export($this->options['hours'], true)) . '"',
			'data-hours-label="' . esc_attr($this->options['hours_label']) . '"',
			'data-minutes="' . esc_attr(var_export($this->options['minutes'], true)) . '"',
			'data-minutes-label="' . esc_attr($this->options['minutes_label']) . '"',
			'data-seconds="' . esc_attr(var_export($this->options['seconds'], true)) . '"',
			'data-seconds-label="' . esc_attr($this->options['seconds_label']) . '"'
		];
		$this->options['atts'] = implode(' ', $payload);
	}
}