<?php
/**
 * @package         FirePlugins Framework
 * @version         1.0.20
 * 
 * @author          FirePlugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2021 FirePlugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework\Base\Widgets;

if (!defined('ABSPATH'))
{
	exit; // Exit if accessed directly.
}

/**
 *  Notification
 */
class Notification extends Widget
{
	/**
	 * Widget default options
	 *
	 * @var array
	 */
	protected $widget_options = [
		/**
		 * The type.
		 * 
		 * Available values:
		 * - information
		 * - success
		 * - error
		 * - warning
		 */
		'type' => 'information',

		/**
		 * The theme.
		 * 
		 * Available values:
		 * 
		 * - theme1
		 * - theme2
		 * - theme3
		 */
		'theme' => 'theme1',

		/**
		 * Whether to display the title.
		 * 
		 * If null, the theme provides the value.
		 */
		'enable_title' => null,

		// The title
		'title' => '',

		// The message
		'message' => '',

		/**
		 * The skin.
		 * 
		 * Available values:
		 * 
		 * - light
		 * - color
		 * - dark
		 * - custom (Allows to select custom text and background color)
		 */
		'skin' => 'light',

		/**
		 * The border radius.
		 * 
		 * If null, the theme provides the value.
		 */
		'border_radius' => null,

		/**
		 * Whether to display the close button or not.
		 * 
		 * If null, the theme provides the value.
		 */
		'show_close_button' => null,

		// The text color
		'text_color' => null,

		// The background color
		'background_color' => null,

		// The CSS variables
		'css_vars' => ''
	];

	/**
	 * Class constructor
	 *
	 * @param array $options
	 */
	public function __construct($options = [])
	{
		parent::__construct($options);

		$this->options['css_class'] .= ' ' . $this->options['theme'] . ' type-' . $this->options['type'] . ' ' . $this->options['skin'];

		// On readonly, do not allow interaction (closing of notification via close button)
		if ($this->options['readonly'])
		{
			$this->options['css_class'] .= ' readonly';
		}

		if (is_null($this->options['enable_title']))
		{
			if (in_array(['theme2', 'theme3'], $this->options['theme']))
			{
				$this->options['enable_title'] = true;
			}
		}

		$this->setCSSVars();
	}

	/**
	 * Sets CSS variables.
	 * 
	 * @return  void
	 */
	private function setCSSVars()
	{
		$css_vars = [];

		// Custom colors apply only to custom skin
		if ($this->options['skin'] === 'custom')
		{
			if (!empty($this->options['text_color']))
			{
				$css_vars['text-color'] = $this->options['text_color'];
			}

			if (!empty($this->options['background_color']))
			{
				$css_vars['background-color'] = $this->options['background_color'];
			}
		}

		if ($this->options['border_radius'] !== 0)
		{
			$css_vars['border-radius'] = $this->options['border_radius'] . 'px !important';
		}

		if (empty($css_vars))
		{
			return;
		}

		if (!$css = \FPFramework\Helpers\CSS::cssVarsToString($css_vars, '.fpf-notification.' . $this->options['id']))
		{
			return;
		}

		$this->options['css_vars'] = $css;
	}
}