<?php
/**
 * @package         Fire Plugins Framework
 * @version         1.0.20
 * 
 * @author          Fire Plugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2021 Fire Plugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework {

	if (!defined('ABSPATH'))
	{
		exit; // Exit if accessed directly.
	}

	use FPFramework\API\API;
	use FPFramework\Base\Renderer;
	use FPFramework\Base\HelperMiddleware;
	use FPFramework\Libs\Translations;
	use FPFramework\Libs\SearchDropdownAjax;
	use FPFramework\Libs\Media;

	final class Framework
	{
		/**
		 * Holds the Framework instance.
		 *
		 * @var  Framework  $instance
		 */
		public static $instance = null;
		
		/**
		 * API
		 * 
		 * @var  API  $api
		 */
		public $api;
		
		/**
		 * Renders views.
		 * 
		 * @var  Renderer  $renderer
		 */
		public $renderer;

		/**
		 * THe Plugin name attached to the framework.
		 * 
		 * @var  String  $plugin_name
		 */
		public $plugin_name;

		/**
		 * All Translations.
		 * 
		 * @var  Translations  $translations
		 */
		public $translations;

		/**
		 * Registers and enqueuees styles and scripts.
		 * 
		 * @var  Media  $media
		 */
		public $media;

		/**
		 * Helper Classes
		 * 
		 * @var  HelperMiddleware
		 */
		public $helper;

		/**
		 * Handle Search Dropdown Ajax
		 * 
		 * @var  SearchDropdownAjax
		 */
		public $search_dropdown_ajax;

		private function __construct($plugin_name)
		{
			$this->plugin_name = $plugin_name;
			
			// Translations
			$this->translations = new Translations();

			$this->initConstants();

			// run init
			$this->init();
			$this->admin_init();
		}

		/**
		 * Returns the current plugin name
		 * 
		 * @return  string
		 */
		public function getPluginPage()
		{
			return isset($_GET['page']) ? sanitize_text_field($_GET['page']) : '';
		}

		/**
		 * Returns the instance of the Framework given the ID
		 * 
		 * @param   string     $name
		 *
		 * @return  Framework  An instance of the class.
		 */
		public static function getInstance($id)
		{
			if (is_null(self::$instance))
			{
				self::$instance = new self($id);
			}

			return self::$instance;
		}

		/**
		 * Initializes all Common components used by front-end and back-end.
		 * 
		 * @return  void
		 */
		public function initCommons()
		{
			// Search Dropdown Ajax
			$this->search_dropdown_ajax = new SearchDropdownAjax();

			// API
			$this->api = new API();

			// Renderer
			$this->renderer = new Renderer(FPF_LAYOUTS_DIR);

			// Media
			$this->media = new Media();

			// Helper
			$this->helper = new HelperMiddleware();
		}

		/**
		 * Initialize Framework Costants
		 * 
		 * @return  void
		 */
		public function initConstants()
		{
			// Framework Base
			if (!defined('FPF_BASE'))
			{
				define('FPF_BASE', __FILE__);
			}

			// Framework Folder Path
			if (!defined('FPF_DIR'))
			{
				define('FPF_DIR', plugin_dir_path(__FILE__));
			}

			// Framework Layouts Folder Path
			if (!defined('FPF_LAYOUTS_DIR'))
			{
				define('FPF_LAYOUTS_DIR', plugin_dir_path(__FILE__) . 'Layouts/');
			}

			// Support URL
			if (!defined('FPF_SUPPORT_URL'))
			{
				define('FPF_SUPPORT_URL', FPF_SITE_URL . 'contact');
			}

			// Framework Media URL
			if (!defined('FPF_MEDIA_URL'))
			{
				define('FPF_MEDIA_URL', plugins_url(strtolower($this->plugin_name)) . '/Inc/Framework/media/');
			}
		}

		/**
		 * Initializes all components used by back-end.
		 * 
		 * @return  void
		 */
		public function initBackendComponents()
		{
			// run actions
			$this->handleAdminActions();
		}

		/**
		 * Runs all Admin Actions
		 * 
		 * @return  void
		 */
		private function handleAdminActions()
		{
			// display admin notice
			$adminNotice = new \FPFramework\Libs\AdminNotice();
			add_action('admin_notices', [$adminNotice, 'displayAdminNotice']);
		}

		/**
		 * Initializes Framework with the required components
		 * 
		 * @return  void
		 */
		public function init() {
			$this->initCommons();
		}

		/**
		 * Initializes Framework with the required components for admins
		 * 
		 * @return  void
		 */
		public function admin_init() {
			if (!is_admin())
			{
				return;
			}

			$this->initBackendComponents();
		}

		/**
		 * Retrieves the translation of $text
		 * 
		 * @param  String  $text
		 * @param  String  $fallback
		 * 
		 * @return  String
		 */
		public function _($text, $fallback = null)
		{
			return $this->translations->_($text, $fallback);
		}
	}
}

namespace {
	/**
	 * The function which returns the one Framework instance.
	 * 
	 * @param   string     $plugin_slug
	 * 
	 * @return  Framework
	 */
	function fpframework($plugin_slug = '') {
		return FPFramework\Framework::getInstance($plugin_slug);
	}
}